/////////////////////////////////////
// DS1307.cpp
// part of 720020-1174x (I2C)
//
// Applied Data Systems
// www.applieddata.net
//
// { See DS1307.h for more description and history }
/////////////////////////////////////
#include "ds1307.h"

#define WORD_TO_BCD(x)	((BYTE)(((x)%10) + (((x)/10)*0x10)))
#define	BCD_TO_WORD(x)	((WORD)(((x)&0x0F) + (((x)>>4)*10)))
#define ORIGINYEAR  1980


DS1307::DS1307() : I2C(0xD0)
{
	// the 1307 has a set address of 1101000{x}
	// where {x} is the I2C R/W bit
}

void DS1307::ReadRTC(SYSTEMTIME *time)
{
	DEBUGMSG(TRUE, (_T("+DS1307::ReadRTC\r\n")));

	RTC_TIME	rtc;
	BYTE		data[2];
	BYTE		*prtc		= (BYTE*)&rtc;

	
	// set the address register pointer
	// send the DS1631_ACCESS_CONFIG command
	data[0]		= m_busAddress | I2C_WRITE;	// address as write
	data[1]		= 0x00;						// register address
	Write(data, 2, TRUE, TRUE);

	// tell the device we're going to read
	data[0]		= m_busAddress | I2C_READ;	// address as read
	Write(data, 1, TRUE, FALSE);

	// read data off the bus
	for(int b = 0 ; b < 7 ; b++)
	{
		data[0]		= 0;	// clear the byte so we know the return is good
		Read(data, 1, FALSE, (b < 6) ? FALSE : TRUE);

		*prtc = data[0];
		prtc++;
	}

	// convert the time
	RTCTime2SystemTime(&rtc, time);

	DEBUGMSG(TRUE, (_T("-DS1307::ReadRTC\r\n")));
}

void DS1307::SetRTC(SYSTEMTIME *time)
{
	DEBUGMSG(TRUE, (_T("+DS1307::SetRTC\r\n")));

	RTC_TIME	rtc;
	BYTE		data[2];

	SystemTime2RTCTime(time, &rtc);

	BYTE		*prtc		= (BYTE*)&rtc;

	// begin the "set"
	// send the register address
	data[0]		= m_busAddress | I2C_WRITE;	// address as write
	data[1]		= 0x00;						// register address
	Write(data, 2, TRUE, FALSE);

	// send the rtc data to the bus
	for(int b = 0 ; b < 7 ; b++)
	{
		Write(prtc, 1, FALSE, (b < 6) ? FALSE : TRUE);
		prtc++;
	}

	DEBUGMSG(TRUE, (_T("-DS1307::SetRTC\r\n")));
}

void DS1307::SystemTime2RTCTime(SYSTEMTIME *st, RTC_TIME *rtc)
{
	rtc->YEAR	= WORD_TO_BCD(st->wYear - ORIGINYEAR);
	rtc->MONTH	= WORD_TO_BCD(st->wMonth);
	rtc->DATE	= WORD_TO_BCD(st->wDay);
	rtc->HOUR	= WORD_TO_BCD(st->wHour);
	rtc->MIN	= WORD_TO_BCD(st->wMinute);
	rtc->SEC	= WORD_TO_BCD(st->wSecond);
	rtc->DAY	= WORD_TO_BCD(st->wDayOfWeek + 1);
}

void DS1307::RTCTime2SystemTime(RTC_TIME *rtc, SYSTEMTIME *st)
{
	st->wYear = BCD_TO_WORD(rtc->YEAR) + ORIGINYEAR;
	st->wMonth = BCD_TO_WORD(rtc->MONTH);
	st->wDay = BCD_TO_WORD(rtc->DATE);

	// SystemTime uses 24Hour mode
	if (rtc->HOUR & 0x40)
	{
		// RTC is 12-hour mode
		// convert RTC to 24-hour mode
		rtc->HOUR &= ~0x40;
		if (rtc->HOUR & 0x20)
		{
			// PM
			rtc->HOUR &= ~0x20;
			rtc->HOUR += 0x12;
		}
	}

	// RTC is 24-hour mode
	st->wHour = BCD_TO_WORD(rtc->HOUR);
	st->wMinute = BCD_TO_WORD(rtc->MIN);
	st->wSecond = BCD_TO_WORD(rtc->SEC);

	// wDayOfWeek starts from 0 but rtc starts from 1
	// DAY in RTC 1 equals wDayOfWeek in SystemTime and it is Sunday
	st->wDayOfWeek = BCD_TO_WORD(rtc->DAY) - 1;

	// Set wMilliseconds field to zero
	st->wMilliseconds = 0;
}
