/////////////////////////////////////
// I2C.h
// 720020-1174x
//
// Applied Data Systems
// www.applieddata.net
//
// Description
// -----------
// This is a base class that custom I2C device implementations can derive from.
// For an example of a derived class, see the sample DS1631 thermometer class.
//
// The ADS I2C bus driver on the AGX and BitsyX uses the WriteFile and ReadFile
// APIs to put data on or read data off the bus.  To give control over setting 
// the start and stop conditions, data passed in must be in an IIC_MESSAGE struct
// format.  The data length passed to the actual WriteFile or ReadFile APIs is
// not used, the uDataLength is the value used.  uDataLength is the length of
// bus data bytes only, it is *not* the size of the IIC_MESSAGE struct.
//
// *Any* data written to the bus, including a command to read, must be done with
// WriteFile.  ReadFile is only used to retrieve data.  Due to the variety of
// I2C devices in the market, it is not possible for the driver to know how much 
// data or the data format expected in either a write or a read, so using values
// for the uDataLength that are unexpected by the client device can cause either 
// the driver or bus to go to an unknown state.
//
// History
// -------
// 720020-11741 - Feb 16, 2004 : ctacke
//   - Initial release
// 720020-11742 - Feb 17, 2004 : ctacke
//   - Created generic Read and Write in the base class
/////////////////////////////////////
#ifndef __I2C_H__
#define __I2C_H__

#include <windows.h>

#define I2C_WRITE	0
#define I2C_READ	1

typedef struct __IIC_MESSAGE
{
   BOOL  bCreateStart;
   BOOL  bCreateStop;
   BYTE  *pData;
   ULONG uDataLength;
} IIC_MESSAGE, *PIIC_MESSAGE;

class I2C
{
	protected:
		HANDLE	m_i2c;
		BYTE	m_busAddress;
		BOOL	Write(BYTE *data, INT dataLength, BOOL setStart, BOOL setStop);
		BOOL	Read(BYTE *data, INT dataLength, BOOL setStart, BOOL setStop);

	public:

		I2C(BYTE busAddress);
		~I2C();
};

#endif