/////////////////////////////////////
// I2CTest.cpp
// part of 720020-1174x (I2C)
//
// Applied Data Systems
// www.applieddata.net
//
// This sample application illustrates the use of a
// custom class wrapper for the DS1631 I2C thermometer
// with an ADS BitsyX or AGX device.
//
// { See DS1631.h for more description and history }
/////////////////////////////////////

#include <windows.h>
#include "ds1631.h"
#include "ds1307.h"

void DS1631Test();
void DS1307Test();

int WINAPI WinMain(	HINSTANCE hInstance,
				   HINSTANCE hPrevInstance,
				   LPTSTR    lpCmdLine,
				   int       nCmdShow)
{
//	DS1631Test();
	DS1307Test();

	return 0;
}

void DS1307Test()
{
	DS1307 *rtc = new DS1307();
	
	LPSYSTEMTIME lpst = new SYSTEMTIME;

	for(int i = 0 ; i < 10 ; i++)
	{
		rtc->ReadRTC(lpst);

		//ConvertSystemTime (lpst, &cur_time); 
		RETAILMSG(1,(L" OEMGetRealTime: Y:%u, M:%u, D:%u, H:%u, M:%u, S:%u  \r\n",
					lpst->wYear, lpst->wMonth,lpst->wDay, lpst->wHour, lpst->wMinute,lpst->wSecond));
		
		lpst->wMinute += 1; 

		rtc->SetRTC(lpst);

		Sleep(1000);
	}


	delete lpst;
	delete rtc;
}

void DS1631Test()
{
	DS1631 *thermometer = new DS1631(0x90);

	// read the device config
	BYTE config = 0;
	config = thermometer->ReadConfiguration();
	RETAILMSG(TRUE, (_T("Current config = 0x%02x\r\n"), config));

	Sleep(1000);

	// set a new config
	config = 0x8D;
	RETAILMSG(TRUE, (_T("Setting config = 0x%02x\r\n"), config));
	thermometer->WriteConfiguration(config);

	Sleep(1000);

	// verify the setting worked
	config = thermometer->ReadConfiguration();
	RETAILMSG(TRUE, (_T("Current config = 0x%02x\r\n"), config));

	Sleep(1000);

	for(int i = 0 ; i < 20 ; i++)
	{
		int temp = (int)thermometer->ReadCurrentTempF();

		RETAILMSG(TRUE, (_T("Temp: %iF\r\n"), temp));
		Sleep(1000);
	}

	delete thermometer ;
}